

#include <p18cxxx.h>
#include "io_cfg.h"
#include "typedefs.h"
#include "usb9.h"
#include "usbvcp.h"




rom USB_DEV_DSC vcp_device_dsc =
{
    sizeof(USB_DEV_DSC),    // Size of this descriptor in bytes
    DSC_DEV,                // DEVICE descriptor type
    0x0200,                 // USB Spec Release Number in BCD format
	0x00,
    0x00,                   // Subclass code
    0x00,                   // Protocol code
    EP0_BUFF_SIZE,          // Max packet size for EP0, see usbcfg.h

	0x0403,                 // Vendor ID
    0x6010,                 // Product ID
    0x0500,                 // Device release number in BCD format
    0x01,                   // Manufacturer string index
    0x02,                   // Product string index
    0x03,                   // Device serial number string index
    0x01                    // Number of possible configurations
};



/* Configuration 1 Descriptor */
rom struct
{
    USB_CFG_DSC             cd01;
    USB_INTF_DSC            i01a00;
    USB_EP_DSC              ep01o_i01a00;
    USB_EP_DSC              ep01i_i01a00;
    USB_INTF_DSC            i02a00;
    USB_EP_DSC              ep02o_i01a00;
    USB_EP_DSC              ep02i_i01a00;

}vcp_cfg01 =
{
    /* Configuration Descriptor */
    sizeof(USB_CFG_DSC),    // Size of this descriptor in bytes
    DSC_CFG,                // CONFIGURATION descriptor type
    sizeof(vcp_cfg01),      // Total length of data for this cfg
    2,                      // Number of interfaces in this cfg
    1,                      // Index value of this configuration
    0,                      // Configuration string index
    _DEFAULT,               // Attributes, see usbdefs_std_dsc.h
    0xFA,                   // Max power consumption (2X mA)
    

	/* Interface Descriptor */
    sizeof(USB_INTF_DSC),   // Size of this descriptor in bytes
    DSC_INTF,               // INTERFACE descriptor type
    0,                      // Interface Number
    0,                      // Alternate Setting Number
    2,                   	// Number of endpoints in this intf
    0xFF,              		// Class code
    0xFF, 					// Subclass code
    0xFF,					// Protocol code
    2                      // Interface string index

    /* Endpoint Descriptors */
	,sizeof(USB_EP_DSC), DSC_EP, _EP01_IN,  _BULK,VCP_BULK_IN_EP_SIZE,  0x00
    ,sizeof(USB_EP_DSC), DSC_EP, _EP01_OUT, _BULK,VCP_BULK_OUT_EP_SIZE, 0x00
    
	,
    /* Interface2 Descriptor */
    sizeof(USB_INTF_DSC),   // Size of this descriptor in bytes
    DSC_INTF,               // INTERFACE descriptor type
    1,                      // Interface Number
    0,                      // Alternate Setting Number
    2,                   	// Number of endpoints in this intf
    0xFF,              		// Class code
    0xFF, 					// Subclass code
    0xFF,					// Protocol code
    2                      // Interface string index

    /* Endpoint Descriptors */
    ,sizeof(USB_EP_DSC), DSC_EP, _EP02_IN,  _BULK,VCP_BULK_IN_EP_SIZE,  0x00
    ,sizeof(USB_EP_DSC), DSC_EP, _EP02_OUT, _BULK,VCP_BULK_OUT_EP_SIZE, 0x00
};


#pragma udata overlay usbram5a  =0x500    //See Linker Script,usb5:0x500-...
volatile far VCP_EPS vcpA_ep;
volatile far VCP_EPS vcpB_ep;
#pragma udata overlay usbram6a  =0x600    //See Linker Script,usb5:0x500-...
volatile far char vcpctl_ep_tx[64];
volatile far char vcpctl_ep_rx[64];


#pragma udata

VCP vcpA;
VCP vcpB;


#pragma code


void ClearArray( byte* startAdr,byte count);
//{
//	while( count--)
//		*startAdr++ = 0;
//}

#define USBVCP_DEFAULT_LATENCY 16


void USBVCPInitVCP( VCP* vcp)
{
	vcp->Latency = USBVCP_DEFAULT_LATENCY;
	vcp->LatencyCounter = USBVCP_DEFAULT_LATENCY;
	vcp->dte_signals.All = 0;
	
	vcp->EP->tx.dce_signals.All = 0;
	ClearArray( (byte *)&vcp->EP->tx.data, sizeof( vcp->EP->tx.data));
	ClearArray( (byte *)&vcp->EP->rx.data, sizeof( vcp->EP->rx.data));

	vcpA_ep.tx.dce_signals.Cts = 1;
	vcpA_ep.tx.dce_signals.Dsr = 1;
	vcpA_ep.tx.dce_signals.Cd = 1;
	vcpA_ep.tx.dce_signals._Ring = 0;
	
	
	vcp->RxIdx = 0;
	vcp->RxLen = 0;
	vcp->TxIdx = 0;



    VCP_CTL_BD_OUT.Cnt = sizeof(vcpctl_ep_rx);  // Set buffer size
    VCP_CTL_BD_OUT.ADR = (byte*)&vcpctl_ep_rx;  // Set buffer address
    VCP_CTL_BD_OUT.Stat._byte = _USIE|_DAT0|_DTSEN;// Set status

    VCP_CTL_BD_IN.ADR = (byte*)&vcpctl_ep_tx;   // Set buffer size
    VCP_CTL_BD_IN.Stat._byte = _UCPU|_DAT1;    // Set buffer address


    VCPA_DATA_UEP = EP_OUT_IN|HSHK_EN;           // Enable 2 data pipes

    VCPA_BULK_BD_OUT.Cnt = sizeof(vcpA_ep.rx);  // Set buffer size
    VCPA_BULK_BD_OUT.ADR = (byte*)&vcpA_ep.rx;  // Set buffer address
    VCPA_BULK_BD_OUT.Stat._byte = _USIE|_DAT0|_DTSEN;// Set status

    VCPA_BULK_BD_IN.ADR = (byte*)&vcpA_ep.tx;   // Set buffer size
    VCPA_BULK_BD_IN.Stat._byte = _UCPU|_DAT1;    // Set buffer address


	vcp->BD_IN = &VCPA_BULK_BD_IN;
	vcp->BD_OUT = &VCPA_BULK_BD_OUT;
}

void USBVCPInitEP( void)
{
	//
	// Control
	//
    VCP_CTL_UEP = EP_OUT_IN|HSHK_EN;           // 

    VCP_CTL_BD_OUT.Cnt = sizeof(vcpctl_ep_rx);  // Set buffer size
    VCP_CTL_BD_OUT.ADR = (byte*)&vcpctl_ep_rx;  // Set buffer address
    VCP_CTL_BD_OUT.Stat._byte = _USIE|_DAT0|_DTSEN;// Set status

    VCP_CTL_BD_IN.ADR = (byte*)&vcpctl_ep_tx;   // Set buffer size
    VCP_CTL_BD_IN.Stat._byte = _UCPU|_DAT1;    // Set buffer address


	//
	// Port A
	//

	vcpA.BD_IN  = &VCPA_BULK_BD_IN;
	vcpA.BD_OUT = &VCPA_BULK_BD_OUT;
	vcpA.EP     = &vcpA_ep;

	vcpA.Latency = USBVCP_DEFAULT_LATENCY;
	vcpA.LatencyCounter = USBVCP_DEFAULT_LATENCY;
	vcpA.dte_signals.All = 0;
	
	vcpA_ep.tx.dce_signals.All = 0;
	ClearArray( (byte *)&vcpA_ep.tx.data, sizeof( vcpA_ep.tx.data));
	ClearArray( (byte *)&vcpA_ep.rx.data, sizeof( vcpA_ep.rx.data));

	vcpA_ep.tx.dce_signals.Cts = 1;
	vcpA_ep.tx.dce_signals.Dsr = 1;
	vcpA_ep.tx.dce_signals.Cd = 1;
	vcpA_ep.tx.dce_signals._Ring = 0;

	
	vcpA.RxIdx = 0;
	vcpA.RxLen = 0;
	vcpA.TxIdx = 0;


    VCP_CTL_UEP = EP_OUT_IN|HSHK_EN;           // 

    VCP_CTL_BD_OUT.Cnt = sizeof(vcpctl_ep_rx);  // Set buffer size
    VCP_CTL_BD_OUT.ADR = (byte*)&vcpctl_ep_rx;  // Set buffer address
    VCP_CTL_BD_OUT.Stat._byte = _USIE|_DAT0|_DTSEN;// Set status

    VCP_CTL_BD_IN.ADR = (byte*)&vcpctl_ep_tx;   // Set buffer size
    VCP_CTL_BD_IN.Stat._byte = _UCPU|_DAT1;    // Set buffer address


    VCPA_DATA_UEP = EP_OUT_IN|HSHK_EN;           // Enable 2 data pipes

    VCPA_BULK_BD_OUT.Cnt = sizeof(vcpA_ep.rx);  // Set buffer size
    VCPA_BULK_BD_OUT.ADR = (byte*)&vcpA_ep.rx;  // Set buffer address
    VCPA_BULK_BD_OUT.Stat._byte = _USIE|_DAT0|_DTSEN;// Set status

    VCPA_BULK_BD_IN.ADR = (byte*)&vcpA_ep.tx;   // Set buffer size
    VCPA_BULK_BD_IN.Stat._byte = _UCPU|_DAT1;    // Set buffer address



	//
	// Port B
	//

	vcpB.BD_IN  = &VCPB_BULK_BD_IN;
	vcpB.BD_OUT = &VCPB_BULK_BD_OUT;
	vcpB.EP     = &vcpB_ep;

	vcpB.Latency = USBVCP_DEFAULT_LATENCY;
	vcpB.LatencyCounter = USBVCP_DEFAULT_LATENCY;
	vcpB.dte_signals.All = 0;
	
	vcpB_ep.tx.dce_signals.All = 0;
	ClearArray( (byte *)&vcpB_ep.tx.data, sizeof( vcpB_ep.tx.data));
	ClearArray( (byte *)&vcpB_ep.rx.data, sizeof( vcpB_ep.rx.data));

	vcpB_ep.tx.dce_signals.Cts = 1;
	vcpB_ep.tx.dce_signals.Dsr = 1;
	vcpB_ep.tx.dce_signals.Cd = 1;
	vcpB_ep.tx.dce_signals._Ring = 0;
	
	
	vcpB.RxIdx = 0;
	vcpB.RxLen = 0;
	vcpB.TxIdx = 0;


    VCPB_DATA_UEP = EP_OUT_IN|HSHK_EN;           // Enable 2 data pipes


    VCPB_BULK_BD_OUT.Cnt = sizeof(vcpB_ep.rx);  // Set buffer size
    VCPB_BULK_BD_OUT.ADR = (byte*)&vcpB_ep.rx;  // Set buffer address
    VCPB_BULK_BD_OUT.Stat._byte = _USIE|_DAT0|_DTSEN;// Set status

    VCPB_BULK_BD_IN.ADR = (byte*)&vcpB_ep.tx;   // Set buffer size
    VCPB_BULK_BD_IN.Stat._byte = _UCPU|_DAT1;    // Set buffer address


}

rom char EEPROM[] = {
					0x08, 0x08,
					0x03, 0x04,
					0x10, 0x60,
					0x00, 0x05,					
					0x80, 0xFA,
					0x08, 0x00,
					0x00, 0x02,
					0x96, 0x0A,
					0xA0, 0x12,
					0xB2, 0x12,
					0x56, 0x00,
					0x00, 0x00,		// 0x0b
					0x00, 0x00,		// 0x0c
					0x00, 0x00,		// 0x0d
					0x00, 0x00,		// 0x0e
					0x00, 0x00,		// 0x0f
					0x00, 0x00,		// 0x10
					0x00, 0x00,		// 0x11
					0x00, 0x00,		// 0x12
					0x00, 0x00,		// 0x13
					0x00, 0x00,		// 0x14
					0x00, 0x00,		// 0x15
					0x00, 0x00,		// 0x16
					0x00, 0x00,		// 0x17
					0x00, 0x00,		// 0x18
					0x00, 0x00,		// 0x19
					0x00, 0x00,		// 0x1A
					0x00, 0x00,		// 0x1B
					0x00, 0x00,		// 0x1C
					0x00, 0x00,		// 0x1D
					0x00, 0x00,		// 0x1E
					0x00, 0x00,		// 0x1F

					0x00, 0x00,		// 0x20
					0x00, 0x00,		// 0x21
					0x00, 0x00,		// 0x22
					0x00, 0x00,		// 0x23
					0x00, 0x00,		// 0x24
					0x00, 0x00,		// 0x25
					0x00, 0x00,		// 0x26
					0x00, 0x00,		// 0x27
					0x00, 0x00,		// 0x28
					0x00, 0x00,		// 0x29
					0x00, 0x00,		// 0x2A
					0x00, 0x00,		// 0x2B
					0x00, 0x00,		// 0x2C
					0x00, 0x00,		// 0x2D
					0x00, 0x00,		// 0x2E
					0x00, 0x00,		// 0x2F


					0x00, 0x00,		// 0x30
					0x00, 0x00,		// 0x31
					0x00, 0x00,		// 0x32
					0x00, 0x00,		// 0x33
					0x00, 0x00,		// 0x34
					0x00, 0x00,		// 0x35
					0x00, 0x00,		// 0x36
					0x00, 0x00,		// 0x37
					0x00, 0x00,		// 0x38
					0x00, 0x00,		// 0x39
					0x00, 0x00,		// 0x3A
					0x00, 0x00,		// 0x3B
					0x00, 0x00,		// 0x3C
					0x00, 0x00,		// 0x3D
					0x00, 0x00,		// 0x3E
					0x00, 0x00,		// 0x3F


					0x00, 0x00,		// 0x40
					0x00, 0x00,		// 0x41
					0x00, 0x00,		// 0x42
					0x00, 0x00,		// 0x43
					0x00, 0x00,		// 0x44
					0x00, 0x00,		// 0x45
					0x00, 0x00,		// 0x46
					0x00, 0x00,		// 0x47
					0x00, 0x00,		// 0x48
					0x00, 0x00,		// 0x49
					0x00, 0x00,		// 0x4a
					0x0A, 0x03,		// 0x4b
					0x46, 0x00,		// 0x4c
					0x54, 0x00,		// 0x4d
					0x44, 0x00,		// 0x4e
					0x49, 0x00,		// 0x4f
					0x12, 0x03,		// 0x50
					0x44, 0x00,		// 0x51
					0x4c, 0x00,		// 0x52
					0x50, 0x00,		// 0x53
					0x32, 0x00,		// 0x54
					0x32, 0x00,		// 0x55
					0x33, 0x00,		// 0x56
					0x32, 0x00,		// 0x57
					0x4D, 0x00,		// 0x58

					0x12, 0x03,		// 0x59
					0x46, 0x00,		// 0x5a
					0x54, 0x00,		// 0x5b
					0x50, 0x00,		// 0x5c
					0x5b, 0x00,		// 0x5d
					0x37, 0x00,		// 0x5e
					0x32, 0x00,		// 0x5f

					0x50, 0x00,		// 0x60
					0x4c, 0x00,		// 0x61
					0x00, 0x00,		// 0x62
					0x01, 0x00,		// 0x63
					0x00, 0x00,		// 0x64
					0x00, 0x00,		// 0x65
					0x00, 0x00,		// 0x66
					0x00, 0x00,		// 0x67
					0x00, 0x00,		// 0x68
					0x00, 0x00,		// 0x69
					0x00, 0x00,		// 0x6A
					0x00, 0x00,		// 0x6B
					0x00, 0x00,		// 0x6C
					0x00, 0x00,		// 0x6D
					0x00, 0x00,		// 0x6E
					0x00, 0x00,		// 0x6F


					0x00, 0x00,		// 0x70
					0x00, 0x00,		// 0x71
					0x00, 0x00,		// 0x72
					0x00, 0x00,		// 0x73
					0x00, 0x00,		// 0x74
					0x00, 0x00,		// 0x75
					0x00, 0x00,		// 0x76
					0x00, 0x00,		// 0x77
					0x00, 0x00,		// 0x78
					0x00, 0x00,		// 0x79
					0x00, 0x00,		// 0x7A
					0x00, 0x00,		// 0x7B
					0x00, 0x00,		// 0x7C
					0x00, 0x00,		// 0x7D
					0x00, 0x00,		// 0x7E

					0x07, 0x99		// 0x7F
};



void USBVCPReqHandler(void)
{
	VCP* pvcp;
	VCP_EPS* peps;

    /*
     * If request recipient is not the device then return
     */

    if(SetupPkt.Recipient != RCPT_DEV) return;

    /*
     * If request type is not class-specific then return
     */


    if(SetupPkt.RequestType != VENDOR ) return;
    

    /*
     * Interface ID must match interface numbers associated with
     * VCP class, else return
     */
	pvcp = &vcpA;
	peps = &vcpA_ep;

    if( (SetupPkt.bIntfID == 1) )
	{
		pvcp = &vcpA;
		peps = &vcpA_ep;
	}
	else
	    if( (SetupPkt.bIntfID == 2) )
		{
			pvcp = &vcpB;
			peps = &vcpB_ep;
		}
		else				// It's probly a EEPROM request
		{
				
		}

	
	switch( SetupPkt.bRequest)
    {
	case 0x00:						// 0x00
		break;

	case VCPR_SET_DTE_SIGNALS:		// 0x01
		pvcp->dte_signals.All = SetupPkt.W_Value._word;
		break;

	case 0x02:						// 0x02
		break;

	case VCPR_SET_BAUDRATE:			// 0x03
		pvcp->Baudrate = SetupPkt.wValue;
		break;

	case VCPR_SET_PARITY_DATABITS:	// 0x04
		pvcp->Databits = SetupPkt.W_Value.byte0;
//		pvcp->Baudrate = SetupPkt.W_Value.byte1;
		break;

	case VCPR_GET_DCE_SIGNALS:		// 0x05
        usb9.ctrl_trf.pSrc.bRam = (byte*)& peps->tx.dce_signals;
        usb9.ctrl_trf.mem       = _RAM;
        LSB(usb9.ctrl_trf.wCount)    = sizeof( peps->tx.dce_signals);
		break;

	case VCPR_FLUSH_DATA:			// 0x06
		pvcp->LatencyCounter = 0;
		break;

	case VCPR_SET_LATENCY:			// 0x09		Value in mSec 0..255, default 16
		pvcp->Latency = SetupPkt.W_Value.byte0;
		pvcp->LatencyCounter = pvcp->Latency;
		break;

	case VCPR_READ_EEPROM:			// 0x90 READ EEPROM
        usb9.ctrl_trf.pSrc.bRom   = (rom byte *) &EEPROM[(SetupPkt.wIndex<<1)];
        usb9.ctrl_trf.mem         = _ROM;
        LSB(usb9.ctrl_trf.wCount) = 2;
		break;
	case VCPR_WRITE_EEPROM:			// 0x91 WRITE EEPROM, not impl
		break;
	}

    usb9.ctrl_trf.session_owner = MUID_VCP;
}




/*
15,14 = 00  sub-integer divisor = 0
15,14 = 01  sub-integer divisor = 0.5
15,14 = 10  sub-integer divisor = 0.25
15,14 = 11  sub-integer divisor = 0.125
The divisor can be extracted for each entry in a few simples steps, as shown here for the
Step 1  re-order the bytes: 9c,80 -> 809c Hex
Step 2  extract the sub-integer divisor; 15 = 1, 14 = 0 -> sub-integer = 0.25
Step 3  extract the integer divisor: 13:0 = 009c Hex = 156 Dec
Step 4 - combine the integer and sub-integer divisors: 156.25 Dec
Step 5  divide 3M by the divisor: 3,000,000/156.25 = 19,200 baud

		switch( SetupPkt.wValue)
		{
		case 0x4138: cdc.Baudrate =	9600;
		case 0x0271: cdc.Baudrate =	4800;
		case 0x04E2: cdc.Baudrate =	2400;
		case 0x09C4: cdc.Baudrate =	1200;
		}

	case 4:						// Set parity/ data bits
		switch( SetupPkt.wValue)
		{
		case 0x0008: "n,8,1";
		case 0x0208: "e,8,1";
		}
*/



rom char USB_SerialNr_StrDsc[] = "FTPX72PL";



void USBVCPGetDscHandler(void)
{
	usb9.ctrl_trf.mem = _ROM;                       // Set memory type

	switch(SetupPkt.bDscType)
	{
	case DSC_DEV:
	    usb9.ctrl_trf.session_owner = MUID_USB9;
	    usb9.ctrl_trf.pSrc.bRom = (rom byte*)&vcp_device_dsc;
	    usb9.ctrl_trf.wCount._word = sizeof(vcp_device_dsc);  
	    break;
	case DSC_CFG:
	    usb9.ctrl_trf.session_owner = MUID_USB9;
	    usb9.ctrl_trf.pSrc.bRom = (rom byte*)&vcp_cfg01;		//*(USB_CD_Ptr+SetupPkt.bDscIndex);
	    usb9.ctrl_trf.wCount._word = *(usb9.ctrl_trf.pSrc.wRom+1);              // Set data count
	    break;
	case DSC_STR:
	    usb9.ctrl_trf.session_owner = MUID_USB9;
		switch( SetupPkt.bDscIndex)
		{
		case 1:		usb9.ctrl_trf.pSrc.bRom = (rom byte *)&USB_Manufactor_StrDsc; break;
		case 2:		usb9.ctrl_trf.pSrc.bRom = (rom byte *)&USB_Product_StrDsc; break;
		case 3:		usb9.ctrl_trf.pSrc.bRom = (rom byte *)&USB_SerialNr_StrDsc; break;
		}
	    break;
	}
}





void USBVCPInit( void)
{
	USBInit( UCFG_VAL);

	usb9.class.InitEP = USBVCPInitEP;
	usb9.class.ReqHandler = USBVCPReqHandler;
	usb9.class.GetDscHandler = USBVCPGetDscHandler;

#if CPU_FREQ == 48
	T0CON = 0b10000111;		// On, 16bits, Fosc, Ps 256 for Fosc=48Mhx
#elif CPU_FREQ == 12
	T0CON = 0b10000101;		// On, 16bits, Fosc, Ps 64 for Fosc=12Mhx
#else
#error Bad _CPU_FREQ for TMR0 init
#endif

	INTCONbits.TMR0IF = 0;
	TMR0H = 255;		// +/- 1mSec
	TMR0L = 209;
	INTCONbits.TMR0IE = 0;  // No interrupt
}




void USBVCPTxRxService( VCP* vcp);


void USBVCPCheckTimer( void)
{
	if( INTCONbits.TMR0IF)
	{
		TMR0H = 255;		// 1mSec
		TMR0L = 209;

		INTCONbits.TMR0IF = 0;
		if( vcpA.LatencyCounter>0)
			--vcpA.LatencyCounter;
		if( vcpB.LatencyCounter>0)
			--vcpB.LatencyCounter;
	}	
}


void USBVCPTasks( void)
{
    USBCheckBusStatus();                    // Must use polling method
    if( PIE2bits.USBIE == 0)
		USBDriverService();                 // Interrupt or polling method
	USBVCPCheckTimer();

	USBVCPTxRxService( &vcpA);
	USBVCPTxRxService( &vcpB);
}

void USBVCPTxRxService( VCP* vcp)
{
	if( vcp->RxIdx == vcp->RxLen)				// Has everything been read by main?
		if( mUSBBufferIsReady( (*vcp->BD_OUT)))	// 
		{
			vcp->RxLen = vcp->BD_OUT->Cnt;
			vcp->RxIdx = 0;

			if( vcp->RxIdx == vcp->RxLen)			// Huh dit begrijp ik niet meer?
		        mUSBReadyBuffer( (*vcp->BD_OUT),  sizeof( VCP_DATA_RX));
		}

    if( mUSBBufferIsReady( (*vcp->BD_IN)))					// Do we own the transmit buffer?
		if( (vcp->TxIdx != 0) || (vcp->LatencyCounter == 0))	// Do we need to transmit it?
		{
	        mUSBReleaseBuffer( (*vcp->BD_IN), vcp->TxIdx + sizeof( VCP_DCE_SIGNALS) );
			vcp->TxIdx = 0;
			vcp->LatencyCounter = vcp->Latency;
		}
	
}


int getcUSBVCP( VCP* vcp)
{
	char c;

    while((usb9.device_state < CONFIGURED_STATE)||(UCONbits.SUSPND==1))
		USBVCPTasks();

	while( vcp->RxIdx == vcp->RxLen)
		USBVCPTasks();

	c = vcp->EP->rx.data[ vcp->RxIdx++];

	if( vcp->RxIdx == vcp->RxLen)
        mUSBReleaseBuffer( (*vcp->BD_OUT), sizeof(vcp->EP->rx));

	return c;
}

int putcUSBVCP( char c, VCP* vcp)
{
//	if(!(vcp->dte_signals.Rts||vcp->dte_signals.Dtr))
//		return c;

    while((usb9.device_state < CONFIGURED_STATE)||(UCONbits.SUSPND==1))
		USBVCPTasks();

	while( vcp->TxIdx == sizeof(vcp->EP->tx.data))
		USBVCPTasks();

	while( !mUSBBufferIsReady( *vcp->BD_IN))
		USBVCPTasks();

	vcp->EP->tx.data[ vcp->TxIdx++] = c;

	if( vcp->TxIdx == sizeof(vcp->EP->tx.data))
	{
        mUSBReleaseBuffer( *vcp->BD_IN, vcp->TxIdx + sizeof( VCP_DCE_SIGNALS) );
		vcp->TxIdx = 0;
		vcp->LatencyCounter = vcp->Latency;
	}

	return c;
}


char kbhitUSBVCP( VCP* vcp)
{
	USBVCPTasks();

    if((usb9.device_state < CONFIGURED_STATE)||(UCONbits.SUSPND==1))
		return 0;

	if( vcp->RxIdx < vcp->RxLen)
		return 1;

	return 0;
}


int putcUSBVCPA( char c) { return putcUSBVCP( c, &vcpA); }
char kbhitUSBVCPA(void)  { return kbhitUSBVCP( &vcpA); }
int getcUSBVCPA(void)    { return getcUSBVCP( &vcpA); }

int putcUSBVCPB( char c) { return putcUSBVCP( c, &vcpB); }
char kbhitUSBVCPB(void)  { return kbhitUSBVCP( &vcpB); }
int getcUSBVCPB(void)    { return getcUSBVCP( &vcpB); }




