
#ifndef USBVCP_H
#define USBVCP_H

/** I N C L U D E S **********************************************************/

/** D E F I N I T I O N S ****************************************************/

#define VCP_BULK_IN_EP_SIZE     64
#define VCP_BULK_OUT_EP_SIZE    64



#define VCP_CTL_INTF_ID         0x00
#define VCP_CTL_UEP             UEP3
#define VCP_CTL_BD_OUT          ep3Bo
#define VCP_CTL_BD_IN			ep3Bi

#define VCP_DATA_INTF_ID        0x01

#define VCPA_DATA_UEP            UEP1
#define VCPA_BULK_BD_OUT         ep1Bo
#define VCPA_BULK_BD_IN          ep1Bi
#define VCPB_DATA_UEP            UEP2
#define VCPB_BULK_BD_OUT         ep2Bo
#define VCPB_BULK_BD_IN          ep2Bi




/* bRequests */

#define VCPR_SET_DTE_SIGNALS		0x01
												// init?
#define VCPR_SET_BAUDRATE			0x03
#define VCPR_SET_PARITY_DATABITS	0x04
#define VCPR_GET_DCE_SIGNALS		0x05
#define VCPR_FLUSH_DATA				0x06		// wild guess
#define VCPR_SET_LATENCY			0x09


#define VCPR_READ_EEPROM			0x90		// read/write eeprom
#define VCPR_WRITE_EEPROM			0x91		// tot eeprom size=128, a read/write does 2 bytes a time




/** S T R U C T U R E S ******************************************************/



typedef union _VCP_DTE_SIGNALS
{
	word All;
	struct
	{
		unsigned Dtr:1;					// Not sure about which Dtr/Dte are the correct ones..
		unsigned Rts:1;
		unsigned :6;
		unsigned Dtr2:1;
		unsigned Rts2:1;
		unsigned :6;
	};
} VCP_DTE_SIGNALS;

 
typedef union _VCP_DCE_SIGNALS
{
	word All;

	struct
	{
		unsigned :4;
		unsigned Cts:1;
		unsigned Dsr:1;
		unsigned _Ring:1;
		unsigned Cd:1;

		unsigned :1;
		unsigned OverRunError:1;
		unsigned ParityError:1;
		unsigned FramingError:1;
		unsigned Break:1;
		unsigned :3;
	};
} VCP_DCE_SIGNALS;



typedef struct _VCP_DATA_RX
{
	char data[ VCP_BULK_OUT_EP_SIZE];
} VCP_DATA_RX;


typedef struct _VCP_DATA_TX
{
	VCP_DCE_SIGNALS dce_signals;
	char data[ VCP_BULK_OUT_EP_SIZE - sizeof( VCP_DCE_SIGNALS)];
} VCP_DATA_TX;

typedef struct _VCP_EPS
{
	VCP_DATA_RX rx;
	VCP_DATA_TX tx;
} VCP_EPS;


/** E X T E R N S ************************************************************/



/** call backs for USB9********************************************************/

void USBVCPInitEP( void);
void USBVCPReqHandler( void);
void USBVCPGetDscHandler( void);

/** P U B L I C  P R O T O T Y P E S *****************************************/


void USBVCPInit( void);
void USBVCPTasks(void);


int getcUSBVCPA(void);
int putcUSBVCPA( char c);
char kbhitUSBVCPA(void);

int getcUSBVCPB(void);
int putcUSBVCPB( char c);
char kbhitUSBVCPB(void);



typedef struct _VCP
{
	char Latency;
	word LatencyCounter;
	word Baudrate;
	char Parity;
	char Databits;
	char RxIdx;
	char RxLen;
	char TxIdx;
	VCP_DTE_SIGNALS dte_signals;
	VCP_EPS* EP;
	BDT* BD_IN;
	BDT* BD_OUT;
} VCP;


extern VCP vcpA;
extern VCP vcpB;


extern rom char USBVCP_Manufactor_StrDsc[];
extern rom char USBVCP_Product_StrDsc[];


extern volatile far VCP_EPS vcpA_ep;
extern volatile far VCP_EPS vcpB_ep;


#endif //
